@@ -17,7 +17,8 @@
 #include <sys/types.h>
 #include "userconfig.h"
 
-typedef enum { error = -3, not_found, no_access, no, yes } t_fsbool;
+typedef enum { fb_error = -3, fb_not_found, fb_no_access, fb_no, fb_yes } t_fs_bool;
+typedef enum { ft_error = -3, ft_not_found, ft_no_access, ft_file, ft_dir, ft_other } t_fs_type;
 
 typedef struct type_filelist {
 	char   *name;
@@ -30,16 +31,16 @@ typedef struct type_filelist {
 
 off_t filesize(char *filename);
 char *make_path(char *dir, char *file);
-t_fsbool contains_not_allowed_symlink(char *filename, char *webroot);
-t_fsbool is_directory(char *file);
-t_fsbool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
+t_fs_bool contains_not_allowed_symlink(char *filename, char *webroot);
+t_fs_bool can_execute(char *file, uid_t uid, gid_t gid, t_groups *groups);
+t_fs_type file_type(char *file);
 int create_file(char *directory, mode_t mode, uid_t uid, gid_t gid);
 int create_directory(char *directory, mode_t mode, uid_t uid, gid_t gid);
 int wipe_directory(char *directory, char *filter);
 int gzip_file(char *src, char *dest);
 int  if_modified_since(char *file, char *datestr);
 FILE *fopen_neighbour(char *filename, char *mode, char *neighbour);
-t_filelist *read_filelist(char *directory);
+t_filelist *read_filelist(char *directory, bool include_hidden_files);
 t_filelist *sort_filelist(t_filelist *filelist);
 void remove_filelist(t_filelist *filelist);
 int write_buffer(int handle, const char *buffer, long size);
